PROGRAM CONRED42;
{$F+}
{***********************************************************
"PUENTE DE TIRISTORES CON INDUCTANCIA SERIE"
Autores: JRomero - Arodri
Fecha: 16-02-98

		 La simulacin corresponde a un PUENTE DE TIRISTORES TRIFASICO
		 CON INDUCTANCIA SERIE. DETALLE DE LA CONMUTACION.

		 y se observarn los siguientes parmetros:

		 * tension de salida. area 'perdida' por la conmutacion.
		 * corriente de los TIRISTORES involucrados en la conmutacion T1 y T3
**********************************************************}

USES
wincrt,xmatdefs, wingraph,Dibus, FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01;

CONST w=PI*100;           {frecuencia de red=50 hz}
		periodo=20e-3;      {periodo=2*PI/w}
		Npuntos=1024;       {nmero de puntos de muestreo}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}


var
	L_conmu, Alpha:NReal; {.Para controlar variables.}
	tension, u, Intensidad, grados:NReal;  {variables para visualizar en pantalla}
	grad, enY1, enY2, enY3, enX:Integer;
{----------------------------------------------------------------------}
{******************************************************************
A continuacin: funciones utilizadas para el manejo de los disparos
de tiristores: disparo(k)
******************************************************************}

FUNCTION disparo(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
		disparo:=(vref<=sin(w*t-2*PI/3-(k-1)*PI/3)) and (cos(w*t-2*PI/3-(k-1)*PI/3)>=0);
END;

{----------------------------------------------------------------------}
{*********************************************************
A continuacin los procedimientos Cirt1, Cirt2 y Funcsr de
definicin del circuito.
*********************************************************}

PROCEDURE CIRT1;
BEGIN

	  vg(1,6,7, tension);            {R}
	  vg(2,6,8, tension);            {S}
	  vg(3,6,9, tension);     		  {T}

	  L(7,1, L_CONMU);   	{L lnea}
	  L(8,2, L_CONMU);		{L lnea}
	  L(9,3, L_CONMU);		{L lnea}
	  r(7,1,10000);				{r para bobina del primario}
	  r(8,2,10000);				{r para bobina del primario}
	  r(9,3,10000);				{r para bobina del primario}

	  r(4,10,0.01);				{r para medida de la i por tiristores}
	  r(4,11,0.01);				{r para medida de la i por tiristores}
	  r(4,12,0.01);            {r para medida de la i por tiristores}
	  r(0,15,0.01);            {r para medida de la i por tiristores}
	  r(0,14,0.01);            {r para medida de la i por tiristores}
	  r(0,13,0.01);            {r para medida de la i por tiristores}

	  ig(0,4,5, 1);            {carga}
	  R(5,0, 0.1);
END;

{---------------------------------------------------------------------}

PROCEDURE CIRT2;
BEGIN
	  th(1,10,disparo(1));     {T1}
	  th(15,3,disparo(2));     {T2}
	  th(2,11,disparo(3));     {T3}
	  th(13,1,disparo(4));     {T4}
	  th(3,12,disparo(5));     {T5}
	  th(14,2,disparo(6));     {T6}
END;

{---------------------------------------------------------------------}

PROCEDURE FUNCSR(t:NReal);
BEGIN
rf[1]:=sin(w*t);        	{R}
rf[2]:=sin(w*t -2*PI/3);	{S}
rf[3]:=sin(w*t -4*PI/3);	{T}
END;

{---------------------------------------------------------------------}

{********************************************************
A continuacin los procedimientos de inicio, fin  y Pesca
necesarios para Simeep
*********************************************************}

PROCEDURE INICIO;
BEGIN
nn:=15;			{nmero de nodos del circuito sin incluir 0 (Tierra)}
nindep:=4;		{nmero de fuentes independientes}
tinicial:=0.001;   {tiempo incial de cada simulacin}
tfinal:=5e-3; {tiempo final de cada simulacin}

{Valores iniciales de variables de estado.
 Estas se definen en el orden en que aparecen las inductancias
 y Capacitancias en Cirt1}
x0[1]:=0.027692448661;		{corresponde a Lconm R}
x0[2]:=-1.1426359745;		{corresponde a Lconm S}
x0[3]:=1.1149435258;	    	{corresponde a Lconm T}

{valores de precisin de Simeep}
dtmax0:=2.5e-6;
dtmin:=dtmax0/100;
PasoDivisor:=6;
TmaxTmin:=1e3;

{indicamos a TRAXP los tiempos que debe graficar}
traxp.tinicial:=tinicial;
traxp.tfinal:=tfinal;

{En esta seccin, definimos el nmero de pantallas, y los colores,
 en nmero de grillas inciales, y si tenemos alguna superpocicin
 de seales en una misma ventana}

gridx:=8;	{las divisiones en X son 12}
gridy:=8;	{las diviciones en Y son 8}
subplot(1,2); {matriz de pantallas de 1 por 1}

{definiciones de visualizacin para la pantalla 0,est asociada al canal 0}
definaY(0, -100, 100);
titulo('(Azul): tension de salida.  (Amarillo): v de fase RT.  (Rojo): v de fase ST');
xlabel('100');
ylabel('100 V/DIV');
linea0;
grid;
{asociamos otros canales a la pantalla 0}
superponga(2,0);
superponga(3,0);
superponga(4,0);
{definimos otros colores para algunos de los canales}
definacolor(0,1);
definacolor(2,0);
definacolor(3,4);
definacolor(4,14);


{definiciones de visualizacin para la pantalla 1,est asociada al canal 1}
definaY(1, -800, 100);
titulo('(Amarillo): intensidad del T3.    ((Naranja): intensidad T1.');
xlabel('');
ylabel('0.5 A/DIV');
linea0;
grid;
{asociamos otros canales a la pantalla 0}
superponga(1,1);
superponga(5,1);
{definimos otros colores para algunos de los canales}
definacolor(1,14);
definacolor(5,12);
END;

{---------------------------------------------------------------------}

PROCEDURE PESCA;
var   k:integer;
		x1,y1,x2,y2:integer;
		coseno, coseno_absoluto:NReal;
BEGIN
grados:=18000*t; {....Para ver en el display el tiempo y los grados.}
if grados>=360 then grados:=grados-360*Int(t/0.02);

Intensidad:=v[5]/0.1;


			  {Calculo el tiempo de conmutacion Uk}
coseno:=cos(alpha*PI/180)-w*L_conmu*2*intensidad/ (SQRT(3)*tension);
if (coseno>-0.000001) AND (coseno<0.000001) then coseno:=0.000001;
coseno_absoluto:=ABS(coseno);
u:=ArcTan(SQRT(ABS(1-SQR(coseno_absoluto)))/coseno_absoluto);
if coseno<0 then U:=PI-U;
u:=u*180/pi;
u:=u-alpha;
u:=u+u/8;



{...Pinto anotaciones sobre las graficas...}

enX:=PUNTOX+24; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{alpha}ANOTAR_EVENTO(1,30,30+alpha,grados, enx,147,80,150,enY2,150, CHR(224));

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Uu}ANOTAR_EVENTO(2,30+alpha,30+alpha+U,grados, enx,167,170,eny3,170,eny2, 'U');

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Ui}ANOTAR_EVENTO(3,30+alpha,30+alpha+U,grados, enx,377,360,380,310,380, 'U');



{indicar el tiempo de simulacin a el graficador}
traxp.t:=t;
{relacin de seales a graficar con los canales deseados}
trazo(0,v[4]);              { TENSION EN LA CARGA }
trazo(0,0);
eny1:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
{RELLENAR_EVENTO(GRAD1,     GRAD2,    GRADOS,ENX,  y1,   y2,   color)}
RELLENAR_EVENTO(30+alpha, 30+alpha+U, grados,enx, enY1, enY2,    13);

trazo(2,v[5]*0);            { Id = Cte }
trazo(3,v[7]-v[8]);         { TENSION R-T }
eny2:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}
trazo(4,v[9]-v[8]);         { TENSION S-T }
eny3:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}

trazo(1,(v[10]-v[4])*20000-400);    { i T1 durante la conmutacion }
trazo(1,0-400);
trazo(5,(v[12]-v[4])*19000-700);    { i T3 durante la conmutacion }
trazo(5,0-700);




	{borrado de la pantalla despus de cada simulacin}
	if t >= tfinal then
	begin
	DELAY(100);

		for k:= 0 to 5 do
		begin
		borrecanal(k,0);
		desligue(k);
		borde;
		grid;
		linea0;
		end;
		t:= t-tfinal+tinicial;

		x0[1]:=0.027692448661;		{corresponde a Lconm R}
		x0[2]:=-1.1426359745;		{corresponde a Lconm S}
		x0[3]:=1.1149435258;		{corresponde a Lconm T}

	end;
END;

{---------------------------------------------------------------------}

PROCEDURE FIN;
begin
	halt;
end;

{---------------------------------------------------------------------}

PROCEDURE INICIALIZACION_PARAMETROS;
BEGIN
{valores iniciales de los controles}
tension:=311;
END;


{----------------------------------------------------------------------}
{******************************PRINCIPAL*******************************}
{----------------------------------------------------------------------}
BEGIN
INICIALIZACION_PARAMETROS;
{Indicamos a Simeep donde estn las funciones que preparamos}
FilUSR.Cirt1:=Cirt1;
FilUSR.Cirt2:=Cirt2;
FilUSR.FuncsR:=FuncsR;
FilUSR.Inicio:=Inicio;
FilUSR.Pesca:=Pesca;
FilUSR.Fin:=Fin;

{Pantalla de controles}
UCntrl.init;

{------------------PARA VISUALIZAR TEXTO EN LA SIMULACION--------------}

UCntrl.insertdisplay(new(PCtrlVerTexto,init
(    0,   '    RECTIFICADOR ',   'PUENTE DE TIRISTORES',
			 'DETALLE  CONMUTACION')));

UCntrl.insert(new(PCtrlVerTexto,init
(    1,   '    ',
			 '        CARGA',
			 '    FUENTE I=Cte.' )));

UCntrl.insert(new(PCtrlVerTexto,init
(    3,   '    ',
			 '    VISUALIZACION    ',
			 '    DE  VARIABLES ')));


UCntrl.insert(new(PCtrlVerTexto,init
(    6,   '    ',
			 '     ',
			 'INDUC.SERIE Y DISPARO ')));


{---------------PARA CONTROLAR VARIABLES EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    7, ' L_CONMU',  'mH'    ,0,       1,  80,  50,  5, L_CONMU)));

UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    8, '  Alpha',  ''    ,0,      1,   50,   20,  2,   alpha)));

{-----------------PARA VER VARIABLES EN LA SIMULACION------------------}

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon,  nombre1,  uni1, deci1,  var1 , nombre2,  uni2, deci2, var2}
(    4,    'Tiempo',   'Sg',   3  ,  t ,  'Grados',  '',    0 ,Grados)));

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    5, 'u',  '',     1  , U , 'Ud', 'V',   1 , v[4])));
{----------------------------------------------------------------------}
{Inicio de Simulaciones}
PASEO;
END.
